@@ -44,8 +44,7 @@ def scan_api(controller, path=[]):
         a = getattr(controller, name)
         if inspect.ismethod(a):
             if hasattr(a, '_wsme_definition'):
-                a._wsme_definition.path = path
-                yield a._wsme_definition
+                yield path + [name], a._wsme_definition
         elif inspect.isclass(a):
             continue
         else:
@@ -98,9 +97,6 @@ class FunctionDefinition(object):
         #: Make sense only with :attr:`protocol_specific` functions.
         self.contenttype = None
 
-        #: Path of the function in the api tree.
-        self.path = None
-
         #: Dictionnary of protocol-specific options.
         self.extra_options = {}
 
@@ -204,7 +200,7 @@ class validate(object):
             args = args[1:]
         for i, argname in enumerate(args):
             datatype = self.param_types[i]
-            mandatory = defaults is None or i <= len(defaults)
+            mandatory = defaults is None or i < (len(args) - len(defaults))
             default = None
             if not mandatory:
                 default = defaults[i - (len(args) - len(defaults))]
@@ -213,6 +209,15 @@ class validate(object):
         return func
 
 
+class CallContext(object):
+    def __init__(self, request):
+        self.request = request
+        self.path = None
+
+        self.func = None
+        self.funcdef = None
+
+
 class WSRoot(object):
     """
     Root controller for webservices.
@@ -232,7 +237,7 @@ class WSRoot(object):
     def addprotocol(self, protocol, **options):
         """
         Enable a new protocol on the controller.
-        
+
         :param protocol: A registered protocol name or an instance
                          of a protocol.
         """
@@ -244,8 +249,8 @@ class WSRoot(object):
     def getapi(self):
         """
         Returns the api description.
-        
-        :rtype: list of :class:`FunctionDefinition`
+
+        :rtype: list of (path, :class:`FunctionDefinition`)
         """
         if self._api is None:
             self._api = [i for i in scan_api(self)]
@@ -287,49 +292,53 @@ class WSRoot(object):
                 res.body = msg
                 log.error(msg)
                 return res
-            path = protocol.extract_path(request)
-            if path is None:
+
+            context = None
+            calls = list(protocol.iter_calls(request))
+
+            if len(calls) != 1:
+                raise NotImplementedError("Batch calls are not yet supported")
+
+            context = calls[0]
+            context.path = protocol.extract_path(context)
+            if context.path is None:
                 raise exc.ClientSideError(
                     u'The %s protocol was unable to extract a function '
                     u'path from the request' % protocol.name)
-            func, funcdef = self._lookup_function(path)
-            kw = protocol.read_arguments(funcdef, request)
+            context.func, context.funcdef = self._lookup_function(context.path)
+            kw = protocol.read_arguments(context)
 
-            for arg in funcdef.arguments:
+            for arg in context.funcdef.arguments:
                 if arg.mandatory and arg.name not in kw:
                     raise exc.MissingArgument(arg.name)
 
-            result = func(**kw)
+            result = context.func(**kw)
 
             res.status = 200
 
-            if funcdef.protocol_specific and funcdef.return_type is None:
+            if context.funcdef.protocol_specific and context.funcdef.return_type is None:
                 if isinstance(result, unicode):
                     res.unicode_body = result
                 else:
                     res.body = result
             else:
                 # TODO make sure result type == a._wsme_definition.return_type
-                res.body = protocol.encode_result(funcdef, result)
-            res_content_type = funcdef.contenttype
+                res.body = protocol.encode_result(context, result)
+            res_content_type = context.funcdef.contenttype
         except Exception, e:
             infos = self._format_exception(sys.exc_info())
             if isinstance(e, exc.ClientSideError):
                 res.status = 400
             else:
                 res.status = 500
-            res.body = protocol.encode_error(infos)
+            res.body = protocol.encode_error(context, infos)
 
         if res_content_type is None:
             # Attempt to correctly guess what content-type we should return.
             last_q = 0
-            if hasattr(request.accept, '_parsed'):
-                for mimetype, q in request.accept._parsed:
-                    if mimetype in protocol.content_types and last_q < q:
-                        res_content_type = mimetype
-            else:
-                res_content_type = request.accept.best_match([
-                    ct for ct in protocol.content_types if ct])
+            ctypes = [ct for ct in protocol.content_types if ct]
+            if ctypes:
+                res_content_type = request.accept.best_match(ctypes)
 
         # If not we will attempt to convert the body to an accepted
         # output format.
